@@ -28,9 +28,10 @@
   (require 'let-alist)
   (require 'eieio))
 
+(require 'pcase)
 (require 'cl-lib)
+(require 'seq)
 (require 'subr-x)
-(require 'dash)
 
 (defvar mini-echo-ellipsis)
 (defvar meow--indicator)
@@ -45,6 +46,7 @@
 (defvar magit-blame-mode-lighter)
 (defvar display-time-string)
 (defvar lsp-bridge-mode-lighter)
+(defvar eglot-mode-line-format)
 (defvar eglot--mode-line-format)
 (defvar envrc--status)
 (defvar flycheck-last-status-change)
@@ -66,11 +68,20 @@
 (defvar profiler-report-order)
 (defvar profiler-report-reversed)
 (defvar xwidget-webkit--loading-p)
+(defvar project-mode-line-format)
+(defvar treesit-explore-mode)
+(defvar treesit-inspect-mode)
+(defvar treesit--inspect-name)
+(defvar org-src--source-type)
+(defvar org-src--babel-info)
 
 (declare-function profiler-profile-type "profiler")
 (declare-function profiler-profile-diff-p "profiler")
 (declare-function eieio-oset "eieio-core")
 (declare-function xwidget-webkit-current-session "xwidget")
+(declare-function xwidget-webkit-estimated-load-progress "xwidget")
+(declare-function xwidget-webkit-title "xwidget")
+(declare-function org-src-edit-buffer-p "org-src")
 (declare-function projectile-project-root "ext:projectile")
 (declare-function ffip-project-root "ext:ffip")
 (declare-function keycast--format "ext:keycast")
@@ -233,6 +244,11 @@ Otherwise, show mise section always."
   "Face for mini-echo segment of blob revision."
   :group 'mini-echo)
 
+(defface mini-echo-special-minor
+  '((t (:inherit mini-echo-magenta-bold)))
+  "Face for mini-echo segment of special minor mode."
+  :group 'mini-echo)
+
 (defface mini-echo-status-local
   '((t (:inherit (mini-echo-yellow bold))))
   "Face for mini-echo segment to show status in local buffer."
@@ -330,37 +346,43 @@ Otherwise, show mise section always."
   "Define a mini echo segment NAME with DOCSTRING and PROPS."
   (declare (indent defun) (doc-string 2))
   (if (plistp props)
-      (-let (((&plist :fetch :setup :update :update-hook :update-advice) props)
-             ((fetch-func update-func setup-func)
-              (--map (intern (concat "mini-echo-segment--" (format "%s-%s" it name)))
-                     '("fetch" "update" "setup"))))
-        `(progn
-           (let ((segment (make-mini-echo-segment :name ,name)))
-             ;; push segment into mini echo alist
-             (setf (alist-get ,name mini-echo-segment-alist nil nil #'string=) segment)
-             (with-slots ((to-fetch fetch) (to-update update)
-                          (to-hook update-hook) (to-advice update-advice)
-                          (to-setup setup))
-                 segment
-               ;; fetch
-               (defun ,fetch-func () ,docstring ,fetch)
-               (setf to-fetch ',fetch-func)
-               ;; update
-               (when (consp ',update)
-                 (defun ,update-func (&rest _args)
-                   (when (bound-and-true-p mini-echo-mode)
-                     ,update))
-                 (setf to-update ',update-func
-                       to-hook ,update-hook
-                       to-advice ,update-advice))
-               ;; setup
-               (and (or ,update-hook ,update-advice (consp ',setup))
-                    (defun ,setup-func ()
-                      ,setup
-                      (--each ,update-hook (add-hook it ',update-func))
-                      (--each ,update-advice (advice-add (car it) (cdr it) ',update-func)))
-                    (setf to-setup ',setup-func)))
-             segment)))
+      (cl-destructuring-bind (&key setup fetch update update-hook update-advice)
+          props
+        (pcase-let* ((`(,fetch-func ,update-func ,setup-func)
+                      (mapcar
+                       (lambda (s)
+                         (intern (concat "mini-echo-segment--"
+                                         (format "%s-%s" s name))))
+                       '("fetch" "update" "setup"))))
+          `(progn
+             (let ((segment (make-mini-echo-segment :name ,name)))
+               ;; push segment into mini echo alist
+               (setf (alist-get ,name mini-echo-segment-alist nil nil #'string=) segment)
+               (with-slots ((to-fetch fetch) (to-update update)
+                            (to-hook update-hook) (to-advice update-advice)
+                            (to-setup setup))
+                   segment
+                 ;; fetch
+                 (defun ,fetch-func () ,docstring ,fetch)
+                 (setf to-fetch ',fetch-func)
+                 ;; update
+                 (when (consp ',update)
+                   (defun ,update-func (&rest _args)
+                     (when (bound-and-true-p mini-echo-mode)
+                       ,update))
+                   (setf to-update ',update-func
+                         to-hook ,update-hook
+                         to-advice ,update-advice))
+                 ;; setup
+                 (and (or ,update-hook ,update-advice (consp ',setup))
+                      (defun ,setup-func ()
+                        ,setup
+                        (dolist (h ,update-hook)
+                          (add-hook h ',update-func))
+                        (dolist (v ,update-advice)
+                          (advice-add (car v) (cdr v) ',update-func)))
+                      (setf to-setup ',setup-func)))
+               segment))))
     (message "mini-echo-define-segment: %s properties error!" name)))
 
 (defun mini-echo-segment--extract (construct &optional force)
@@ -490,25 +512,14 @@ with ellipsis."
 
 (defun mini-echo-buffer-name ()
   "Return current buffer name."
-  (cond
-   (;; TODO support timemachine file
-    (or (bound-and-true-p magit-blob-mode))
-    (save-match-data
-      (let ((str (buffer-name)))
-        (when (string-match "\\(.+\\)\\.~\\(.+\\)~" str)
-          (concat (file-name-nondirectory (match-string 1 str))
-                  (propertize (concat "@" (substring (match-string 2 str) 0 7))
-                              'face 'mini-echo-blob-revision))))))
-   ((bound-and-true-p atomic-chrome-edit-mode)
-    (mini-echo-segment--print (buffer-name) nil 25))
-   (t (let ((uniquify-buffer-name-style 'forward))
-        (buffer-name)))))
+  (let ((uniquify-buffer-name-style 'forward))
+    (buffer-name)))
 
 (defun mini-echo-buffer-name-with-status ()
   "Return last part of buffer name with status."
   ;; NOTE only highlight the last part of buffer-name due to `uniquify-buffer-name-style'
-  (-let* ((name-end (file-name-nondirectory (mini-echo-buffer-name)))
-          ((sign . face) (mini-echo-buffer-status)))
+  (pcase-let* ((name-end (file-name-nondirectory (mini-echo-buffer-name)))
+               (`(,sign . ,face) (mini-echo-buffer-status)))
     (pcase mini-echo-buffer-status-style
       ('sign (concat name-end (propertize sign 'face face)))
       ('color (propertize name-end 'face face))
@@ -531,20 +542,18 @@ with ellipsis."
    (let* ((filepath (buffer-file-name))
           (project (or mini-echo--project-root
                        (mini-echo-update-project-root)))
-          (dir (->> default-directory
-                    (or (and (not (string-empty-p project)) project))
-                    (directory-file-name)
-                    (file-name-nondirectory))))
+          (dir (thread-last default-directory
+                            (or (and (not (string-empty-p project)) project))
+                            (directory-file-name)
+                            (file-name-nondirectory))))
      (cond
       ((not filepath) "")
       ((string-empty-p project) (propertize (concat dir "/") 'face 'shadow))
       ((string-prefix-p project filepath)
-       (let* ((parts (butlast (split-string (string-remove-prefix project filepath) "/")))
-              (suffix (if (<= (length parts) 4)
-                          (string-join `("" ,@(--map (substring it 0 1) parts) "") "/")
-                        "/../")))
-         (concat (propertize dir 'face 'mini-echo-project)
-                 (propertize suffix 'face 'shadow))))
+       (concat (propertize dir 'face 'mini-echo-project) "/"
+               (when-let* ((p (butlast (split-string (string-remove-prefix project filepath) "/" t))))
+                 (concat (propertize (mapconcat (lambda (s) (substring s 0 1)) p "/") 'face 'shadow)
+                  "/"))))
       (t "")))
    (mini-echo-buffer-name-with-status))
   :update (mini-echo-update-project-root))
@@ -682,10 +691,10 @@ Display format is inherited from `battery-mode-line-format'."
                        (propertize "*" 'face 'compilation-mode-line-run)
                        (substring ind 1)))
      (let ((flymake-suppress-zero-counters nil))
-       (string-join (-map #'mini-echo-segment--extract
-                          '(flymake-mode-line-error-counter
-                            flymake-mode-line-warning-counter
-                            flymake-mode-line-note-counter))
+       (string-join (mapcar #'mini-echo-segment--extract
+                            '(flymake-mode-line-error-counter
+                              flymake-mode-line-warning-counter
+                              flymake-mode-line-note-counter))
                     "/")))))
 
 (defsubst mini-echo-column (pos)
@@ -698,10 +707,10 @@ Display format is inherited from `battery-mode-line-format'."
   :fetch
   (when (or mark-active (and (bound-and-true-p evil-local-mode)
                              (eq evil-state 'visual)))
-    (-let [(beg . end)
-           (if (and (bound-and-true-p evil-local-mode) (eq evil-state 'visual))
-               (cons evil-visual-beginning evil-visual-end)
-             (cons (region-beginning) (region-end)))]
+    (pcase-let* ((`(,beg . ,end)
+                  (if (and (bound-and-true-p evil-local-mode) (eq evil-state 'visual))
+                      (cons evil-visual-beginning evil-visual-end)
+                    (cons (region-beginning) (region-end)))))
       (let* ((lines (count-lines beg (min end (point-max))))
              (str (cond ((or (bound-and-true-p rectangle-mark-mode)
                              (and (bound-and-true-p evil-visual-selection)
@@ -762,8 +771,40 @@ Segment appearance depends on var `vc-display-status' and faces like
               (propertize (or progress "") 'face 'mini-echo-yellow-bold)
               (propertize title 'face 'mini-echo-green-bold)))))
 
+(mini-echo-define-segment "org-src"
+  "Return info of `org-src-edit-buffer'."
+  :fetch
+  (when (org-src-edit-buffer-p)
+    (let* ((type (symbol-name org-src--source-type))
+           (lang (car org-src--babel-info))
+           (name (buffer-name)))
+      (save-match-data
+        (when (string-match "\\*Org Src \\([^[]*\\)\\[ .* \\]\\*" name)
+          (setq name (match-string 1 name))))
+      (format "%s|%s|%s %s"
+              (propertize lang 'face 'mini-echo-blue)
+              (propertize type 'face 'mini-echo-yellow)
+              (propertize name 'face 'mini-echo-red)
+              (propertize " ORG-SRC" 'face 'mini-echo-special-minor)))))
+
 ;;; Third-party segments
 
+;; TODO support timemachine file
+
+;; Add more info e.g. steps between blob and head
+(mini-echo-define-segment "magit-blob"
+  "Return info of `magit-blob' file."
+  :fetch
+  (when (bound-and-true-p magit-blob-mode)
+    (save-match-data
+      (let ((str (buffer-name)))
+        (when (string-match "\\(.+\\)\\.~\\(.+\\)~" str)
+          (concat
+           (file-name-nondirectory (match-string 1 str))
+           (propertize (concat "@" (substring (match-string 2 str) 0 7))
+                       'face 'mini-echo-blob-revision)
+           (propertize " Blob" 'face 'mini-echo-special-minor)))))))
+
 (mini-echo-define-segment "flycheck"
   "Return flycheck diagnostics of current buffer."
   :fetch
@@ -775,12 +816,13 @@ Segment appearance depends on var `vc-display-status' and faces like
                         ('running "*")
                         ('finished nil))))
        (propertize ind 'face 'compilation-mode-line-run))
-     (apply #'format "%s/%s/%s"
-            (--zip-with (propertize it 'face other)
-                        (let-alist (flycheck-count-errors flycheck-current-errors)
-                          (--map (number-to-string (or it 0))
-                                 (list .error .warning .info)))
-                        '(error warning success))))))
+     (let* ((errors (flycheck-count-errors flycheck-current-errors)))
+       (setq errors (push (cons 'success (alist-get 'info errors)) errors))
+       (mapconcat
+        (lambda (s)
+          (propertize (number-to-string (or (alist-get s errors) 0)) 'face s))
+        '(error warning success)
+        "/")))))
 
 (mini-echo-define-segment "meow"
   "Return the meow status of current buffer."
@@ -838,7 +880,10 @@ Segment appearance depends on var `vc-display-status' and faces like
   "Return eglot server state"
   :fetch
   (when (bound-and-true-p eglot--managed-mode)
-    (mini-echo-segment--extract eglot--mode-line-format)))
+    (mini-echo-segment--extract
+     (if (boundp 'eglot-mode-line-format)
+         eglot-mode-line-format
+       eglot--mode-line-format))))
 
 (mini-echo-define-segment "envrc"
   "Return envrc status of current buffer."
@@ -880,15 +925,16 @@ Segment appearance depends on var `vc-display-status' and faces like
   "Return info of atomic chrome edit buffers."
   :fetch
   (when (bound-and-true-p atomic-chrome-edit-mode)
-    (propertize "Atomic" 'face 'mini-echo-cyan)))
+    (concat (mini-echo-segment--print (buffer-name) nil 25)
+            (propertize " Atomic" 'face 'mini-echo-special-minor))))
 
 (mini-echo-define-segment "blame"
   "Return info of blame buffers."
   :fetch
   (when (bound-and-true-p magit-blame-mode)
     (if (string-empty-p magit-blame-mode-lighter)
-        (propertize "Blame" 'face 'mini-echo-cyan)
-      (mini-echo-segment--print magit-blame-mode-lighter 'mini-echo-cyan))))
+        (propertize "Blame" 'face 'mini-echo-special-minor)
+      (mini-echo-segment--print magit-blame-mode-lighter 'mini-echo-special-minor))))
 
 (mini-echo-define-segment "helpful"
   "Return info of helpful buffers."
@@ -929,21 +975,23 @@ Segment appearance depends on var `vc-display-status' and faces like
   (when (and (fboundp 'popper-display-control-p)
              (popper-display-control-p (current-buffer)))
     (string-join
-     (->>
-      popper-buried-popup-alist
-      (alist-get (and popper-group-function (funcall popper-group-function)))
-      (-map #'cdr)
-      (cons (current-buffer))
-      (-map #'buffer-name)
-      (-uniq)
-      (-sort #'string-lessp)
-      (-non-nil)
-      (reverse)
-      (--map (mini-echo-segment--print
-              it (if (string= it (buffer-name))
-                     'mini-echo-yellow-bold
-                   'mini-echo-gray)
-              25)))
+     (thread-last
+       popper-buried-popup-alist
+       (alist-get (and popper-group-function (funcall popper-group-function)))
+       (mapcar #'cdr)
+       (cons (current-buffer))
+       (mapcar #'buffer-name)
+       (seq-uniq)
+       (seq-sort #'string-lessp)
+       (seq-filter #'identity)
+       (reverse)
+       (mapcar
+        (lambda (x)
+          (mini-echo-segment--print
+           x (if (string= x (buffer-name))
+                 'mini-echo-yellow-bold
+               'mini-echo-gray)
+           25))))
      (propertize "|" 'face 'font-lock-doc-face))))
 
 (mini-echo-define-segment "wgrep"
@@ -951,7 +999,7 @@ Segment appearance depends on var `vc-display-status' and faces like
   :fetch
   (when (and (bound-and-true-p wgrep-prepared)
              wgrep-sibling-buffer)
-    (propertize "Wgrep" 'face 'mini-echo-yellow-bold)))
+    (propertize "Wgrep" 'face 'mini-echo-special-minor)))
 
 ;; TODO add more segments
 
