@@ -4,9 +4,9 @@
 
 ;; Author: Eki Zhang <liuyinz95@gmail.com>
 ;; Maintainer: Eki Zhang <liuyinz95@gmail.com>
-;; Package-Version: 0.15.0
-;; Package-Revision: fe1fe43b3ad6
-;; Package-Requires: ((emacs "29.1") (dash "2.19.1") (hide-mode-line "1.0.3"))
+;; Package-Version: 20251203.1147
+;; Package-Revision: 3a490395652f
+;; Package-Requires: ((emacs "29.1") (hide-mode-line "1.0.3"))
 ;; Keywords: frames
 ;; Homepage: https://github.com/eki3z/mini-echo.el
 
@@ -42,7 +42,6 @@
 (require 'pcase)
 (require 'map)
 
-(require 'dash)
 (require 'hide-mode-line)
 
 (require 'mini-echo-segments)
@@ -52,7 +51,7 @@
   :group 'mini-echo)
 
 (defcustom mini-echo-persistent-rule
-  '(:long ("major-mode" "shrink-path" "vcs" "buffer-position" "buffer-size" "flymake")
+  '(:long ("major-mode" "shrink-path" "vcs" "buffer-position" "buffer-size" "remote-host" "flymake")
     :short ("buffer-name" "buffer-position" "flymake"))
   "Plist of segments which are persistent for buffers.
 Used as fallback if `mini-echo-persistent-function' return nil."
@@ -130,6 +129,13 @@ Format is a list of three argument:
   :type '(symbol number number)
   :group 'mini-echo)
 
+(defcustom mini-echo-skip-update-functions
+  '(next-error-no-select
+    previous-error-no-select)
+  "List of functions which will skip update mini echo during running."
+  :type '(repeat function)
+  :group 'mini-echo)
+
 (defface mini-echo-minibuffer-window
   '((t :inherit default))
   "Face used to highlight the minibuffer window.")
@@ -143,7 +149,8 @@ Format is a list of three argument:
 (defvar mini-echo--default-rule nil)
 (defvar-local mini-echo--selected-rule nil)
 (defvar mini-echo--toggled-segments nil)
-(defvar mini-echo--info-last-built nil)
+(defvar mini-echo--info-cache nil)
+(defvar mini-echo--skip-update nil)
 
 
 ;;; Segments functions
@@ -154,9 +161,13 @@ Format is a list of three argument:
 
 (defun mini-echo-normalize-rule (rule)
   "Return a plist of (:long ... :short ...) according to RULE."
-  (let* ((valid-rule (--map-when (not (keywordp it))
-                                 (-filter #'mini-echo-segment-valid-p it)
-                                 rule))
+  (let* ((valid-rule
+          (mapcar
+           (lambda (s)
+             (if (not (keywordp s))
+                 (seq-filter #'mini-echo-segment-valid-p s)
+               s))
+           rule))
          (use-both (memq :both valid-rule)))
     (list :long (plist-get valid-rule (or (and use-both :both) :long))
           :short (plist-get valid-rule (or (and use-both :both) :short)))))
@@ -175,8 +186,12 @@ Otherwise, return nil."
   (with-current-buffer (current-buffer)
     ;; NOTE return the first match, so the former has higher priority
     (pcase major-mode
+      ((guard (and (fboundp 'org-src-edit-buffer-p) (org-src-edit-buffer-p)))
+       '(:both ("org-src" "buffer-position" "buffer-size" "flymake")))
       ((guard (bound-and-true-p atomic-chrome-edit-mode))
-       '(:both ("atomic-chrome" "buffer-name" "buffer-position" "flymake")))
+       '(:both ("atomic-chrome" "buffer-position" "flymake")))
+      ((guard (bound-and-true-p magit-blob-mode))
+       '(:both ("magit-blob" "buffer-position" "buffer-size")))
       ((guard (or (memq major-mode '(git-commit-elisp-text-mode git-rebase-mode))
                   (string-match-p "\\`magit-.*-mode\\'" (symbol-name major-mode))))
        '(:both ("major-mode" "project")))
@@ -195,7 +210,7 @@ Otherwise, return nil."
 
 (defun mini-echo-ensure ()
   "Ensure all predefined variable ready for mini echo."
-  (setq mini-echo--valid-segments (-map #'car mini-echo-segment-alist))
+  (setq mini-echo--valid-segments (map-keys mini-echo-segment-alist))
   (setq mini-echo--default-rule
         (mini-echo-merge-rules mini-echo-persistent-rule)))
 
@@ -212,36 +227,40 @@ Otherwise, return nil."
                               (if (funcall mini-echo-short-style-predicate)
                                   :short :long)))
            extra)
-       (--each mini-echo--toggled-segments
-         (-let [(segment . enable) it]
+       (dolist (tog mini-echo--toggled-segments)
+         (pcase-let* ((`(,segment . ,enable) tog))
            (if enable
                (unless (member segment result)
                  (push segment extra))
              (setq result (remove segment result)))))
-       (-concat result extra)))
+       (append result extra)))
     ;; FIXME only filter persistent segments
-    ('no-current (-difference mini-echo--valid-segments
-                              (mini-echo-get-segments 'current)))
+    ('no-current (seq-difference mini-echo--valid-segments
+                                 (mini-echo-get-segments 'current)))
     ('toggle (cl-remove-duplicates
-              (-concat (-map #'car mini-echo--toggled-segments)
-                       (mini-echo-get-segments 'current)
-                       (mini-echo-get-segments 'no-current))
+              (append (map-keys mini-echo--toggled-segments)
+                      (mini-echo-get-segments 'current)
+                      (mini-echo-get-segments 'no-current))
               :test #'equal
               :from-end t))))
 
 (defun mini-echo-concat-segments ()
   "Return concatenated information of selected segments."
-  (-> (->> (mini-echo-get-segments 'current)
-           (--map (with-slots (activate setup fetch update)
-                      (alist-get it mini-echo-segment-alist nil nil #'string=)
-                    (unless activate
-                      (setq activate t)
-                      (and setup (funcall setup))
-                      (and update (funcall update)))
-                    (funcall fetch)))
-           (--filter (> (length it) 0))
-           (reverse))
-      (string-join mini-echo-separator)))
+  (string-join
+   (thread-last
+     (mini-echo-get-segments 'current)
+     (mapcar
+      (lambda (s)
+        (with-slots (activate setup fetch update)
+            (alist-get s mini-echo-segment-alist nil nil #'string=)
+          (unless activate
+            (setq activate t)
+            (and setup (funcall setup))
+            (and update (funcall update)))
+          (funcall fetch))))
+     (seq-filter (lambda (s) (> (length s) 0)))
+     (reverse))
+   mini-echo-separator))
 
 (defun mini-echo--toggle-completion ()
   "Return completion table for command mini echo toggle."
@@ -251,8 +270,10 @@ Otherwise, return nil."
       (complete-with-action
        action
        (let ((current (mini-echo-get-segments 'current)))
-         (--map (propertize it 'face (if (member it current) 'success 'error))
-                (mini-echo-get-segments 'toggle)))
+         (mapcar
+          (lambda (s)
+            (propertize s 'face (if (member s current) 'success 'error)))
+          (mini-echo-get-segments 'toggle)))
        string pred))))
 
 
@@ -270,50 +291,65 @@ If optional arg SHOW is non-nil, show the mode line."
     (when (< emacs-major-version 31)
       (run-with-timer
        5 nil
-       (lambda () (when-let* ((bufs (--remove (buffer-local-value 'hide-mode-line-mode it)
-                                              (buffer-list))))
-                    (--each bufs (with-current-buffer it (hide-mode-line-mode 1)))))))))
+       (lambda ()
+         (when-let* ((bufs (seq-remove
+                            (lambda (s)
+                              (buffer-local-value 'hide-mode-line-mode s))
+                            (buffer-list))))
+           (dolist (buf bufs)
+             (with-current-buffer buf
+               (hide-mode-line-mode 1)))))))))
 
 (defun mini-echo-show-divider (&optional hide)
   "Show window divider when enable mini echo.
 If optional arg HIDE is non-nil, disable the mode instead."
   (if hide
       (window-divider-mode -1)
-    (-let [(window-divider-default-places
-            window-divider-default-right-width
-            window-divider-default-bottom-width)
-           mini-echo-window-divider-args]
+    (pcase-let ((`(,window-divider-default-places
+                   ,window-divider-default-right-width
+                   ,window-divider-default-bottom-width)
+                 mini-echo-window-divider-args))
       (window-divider-mode 1))))
 
 (defun mini-echo-fontify-minibuffer-window ()
   "Fontify whole window with user defined face attributes."
   (face-remap-add-relative 'default 'mini-echo-minibuffer-window))
 
+(defun mini-echo-skip-update-functions-advice (orig-func &rest args)
+  (setq mini-echo--skip-update t)
+  (unwind-protect
+      (apply orig-func args)
+    (setq mini-echo--skip-update nil)))
+
 (defun mini-echo-init-echo-area (&optional deinit)
   "Initialize echo area and minibuffer in mini echo.
 If optional arg DEINIT is non-nil, remove all overlays."
   ;; delete old overlays by default
-  (-each mini-echo-overlays #'delete-overlay)
+  (mapc #'delete-overlay mini-echo-overlays)
   (setq mini-echo-overlays nil)
   (if deinit
       (progn
-        (--each mini-echo-modified-buffers
-          (with-current-buffer (get-buffer-create it)
+        (dolist (buf mini-echo-modified-buffers)
+          (with-current-buffer (get-buffer-create buf)
             (when (minibufferp) (delete-minibuffer-contents))
             (face-remap-remove-relative mini-echo--remap-cookie)
             (setq-local mini-echo--remap-cookie nil)))
+        (dolist (func mini-echo-skip-update-functions)
+          (advice-remove func #'mini-echo-skip-update-functions-advice))
         (cancel-function-timers #'mini-echo-update)
         (advice-remove 'message #'mini-echo-update-overlays-before-message)
         (remove-hook 'window-size-change-functions #'mini-echo-update-overlays-when-resized)
         (remove-hook 'minibuffer-inactive-mode-hook #'mini-echo-fontify-minibuffer-window)
         (remove-hook 'minibuffer-setup-hook #'mini-echo-fontify-minibuffer-window))
-    (--each mini-echo-modified-buffers
-      (with-current-buffer (get-buffer-create it)
+    (dolist (buf mini-echo-modified-buffers)
+      (with-current-buffer (get-buffer-create buf)
         (and (minibufferp) (= (buffer-size) 0) (insert " "))
         (push (make-overlay (point-min) (point-max) nil nil t)
               mini-echo-overlays)
         (setq-local mini-echo--remap-cookie
                     (mini-echo-fontify-minibuffer-window))))
+    (dolist (func mini-echo-skip-update-functions)
+      (advice-add func :around #'mini-echo-skip-update-functions-advice))
     ;; FIXME sometimes update twice when switch from echo to minibuf
     (run-with-timer 0 mini-echo-update-interval #'mini-echo-update)
     (advice-add 'message :before #'mini-echo-update-overlays-before-message)
@@ -346,15 +382,15 @@ On the gui, calculate length based on pixel, otherwise based on char."
 (defun mini-echo-build-info ()
   "Build mini-echo information."
   (condition-case nil
-      (if-let* ((win (get-buffer-window))
-                ((window-live-p win)))
+      (if (and (window-live-p (get-buffer-window))
+               (not mini-echo--skip-update))
           (let* ((combined (mini-echo-concat-segments))
                  (padding (+ mini-echo-right-padding
                              (mini-echo-calculate-length combined)))
                  (prop `(space :align-to (- right-fringe ,padding))))
-            (setq mini-echo--info-last-built
+            (setq mini-echo--info-cache
                   (concat (propertize " " 'cursor 1 'display prop) combined)))
-        mini-echo--info-last-built)
+        mini-echo--info-cache)
     (format "mini-echo info building error")))
 
 (defun mini-echo-update-overlays (&optional msg)
@@ -363,9 +399,9 @@ If MSG is nil, then use `current-message' instead."
   (when-let* (((not (active-minibuffer-window)))
               (msg (or msg (current-message) ""))
               (info (mini-echo-build-info)))
-    (--each mini-echo-overlays
-      (overlay-put it 'after-string
-                   (if (or (equal (buffer-name (overlay-buffer it))
+    (dolist (ov mini-echo-overlays)
+      (overlay-put ov 'after-string
+                   (if (or (equal (buffer-name (overlay-buffer ov))
                                   " *Minibuf-0*")
                            (> (- (mini-echo-minibuffer-width)
                                  (string-width info)
@@ -386,9 +422,9 @@ ARGS is optional."
   "Update mini echo info in minibuf and echo area."
   (unless (active-minibuffer-window)
     ;; update echo area overlays after-string only if it's not empty
-    (--each-while mini-echo-overlays
-        (not (string-empty-p (overlay-get it 'after-string)))
-      (overlay-put it 'after-string (mini-echo-build-info)))))
+    (dolist (ov mini-echo-overlays)
+      (unless (string-empty-p (overlay-get ov 'after-string))
+        (overlay-put ov 'after-string (mini-echo-build-info))))))
 
 
 ;;; Commands
